extends MarginContainer

@export var initial_sections_count: int = 1

var sections_container: VBoxContainer

const energy_section_node = preload("res://src/modules/building/energy_calculator/energy_section.tscn")

var placeholder_id = 0;
var sections: Array[EnergySection] = []


func del_section(node: Node):
	for i in range(0, sections.size()):
		var section = sections[i]
		if section == node:
			sections_container.remove_child(node)
			sections.pop_at(i)
			break


func _on_sections_deleted(section_node: Node):
	del_section(section_node)


func save_data():
	var save_data_file = FileAccess.open("user://energy_table.save", FileAccess.WRITE)
	var sections_data = sections.map(func get_saved_data(section): return section.get_save_data())
	var json_string = JSON.stringify(sections_data)
	save_data_file.store_string(json_string)


func _on_energy_section_changed():
	save_data()


func add_section(data_section: Dictionary):
	var instance = energy_section_node.instantiate()
	
	placeholder_id += 1
	if data_section.has("title_placeholder") and data_section.title_placeholder:
		instance.title_placeholder = data_section.title_placeholder
	else:
		instance.title_placeholder = "Ядро %s" % placeholder_id
	
	if data_section.has("title"):
		instance.title = data_section.title
	
	if data_section.has("rows"):
		for row in data_section.rows:
			var d = EnergyTableRowData.new()
			d.building_id = row.building_id
			d.building_count = row.building_count
			instance.rows_data.append(d)

	sections_container.add_child(instance)
	
	instance.add_theme_constant_override("margin_bottom", 9)
	instance.add_to_group("Persist")
	instance.deleted.connect(_on_sections_deleted)
	instance.changed.connect(_on_energy_section_changed)
	sections.append(instance)


func load_data():
	if not FileAccess.file_exists("user://energy_table.save"):
		return

	for section in sections:
		del_section(section)

	var save_data_file = FileAccess.open("user://energy_table.save", FileAccess.READ)
	var file_content = save_data_file.get_as_text()
	
	var json = JSON.new()
	var parse_result = json.parse(file_content)
	
	var sections_data: Array = []
	
	if parse_result == OK:
		# TODO: check array
		sections_data = json.get_data()
	else:
		save_data_file.seek(0)
		var section_rows: Array[Dictionary] = []
		while save_data_file.get_position() < save_data_file.get_length():
			var json_string = save_data_file.get_line()

			json = JSON.new()
			parse_result = json.parse(json_string)
			if not parse_result == OK:
				print("JSON Parse Error: ", json.get_error_message(), " in ", json_string, " at line ", json.get_error_line())
				continue

			var node_data = json.get_data()
			section_rows.append(node_data)
		
		sections_data.append({ "rows": section_rows })

	for data in sections_data:
		add_section(data)


func _enter_tree():
	sections_container = $PanelContainer/MarginContainer/ScrollContainer/VBoxContainer/SectionsVBoxContainer
	
	load_data()
	if sections.size() < initial_sections_count:
		add_section({})


func _on_add_section_button_pressed():
	add_section({})

