class_name EnergyTable
extends Control

signal changed()

@export var initial_table_rows: int = 1


@export var title: String = ""
@export var title_placeholder: String
@export var rows_data: Array[EnergyTableRowData]

const table_row_node = preload("res://src/modules/building/energy_calculator/energy_table_row.tscn")
var table_title_line_edit: LineEdit
var total_energy_label: Label
var table_body: VBoxContainer

var table_rows: Array[EnergyTableRow] = []


func render_total_energy_label():
	var total_energy = 0
	for table_row in table_rows:
		total_energy += table_row.total_energy
	
	total_energy_label.text = str(total_energy)


func del_row(node: Node):
	for i in range(0, table_rows.size()):
		var table_row = table_rows[i]
		if table_row == node:
			table_body.remove_child(node)
			table_rows.pop_at(i)
			break


func _on_table_row_deleted(instance: Node):
	del_row(instance)
	render_total_energy_label()
	changed.emit()


func add_row(data_row: Dictionary):
	var instance = table_row_node.instantiate()
	if data_row.has("building_id"):
		instance.building_id = data_row["building_id"]
	if data_row.has("building_count"):
		instance.building_count = data_row["building_count"]
	
	table_body.add_child(instance)
	
	instance.add_to_group("Persist")
	instance.total_energy_changed.connect(_on_total_energy_changed)
	instance.deleted.connect(_on_table_row_deleted)
	
	table_rows.append(instance)


func _enter_tree():
	table_body = $PanelContainer/MarginContainer/VBoxContainer/TableBody/VBoxContainer
	table_title_line_edit = $PanelContainer/MarginContainer/VBoxContainer/Header/TableTitleLineEdit
	total_energy_label = $PanelContainer/MarginContainer/VBoxContainer/TableSummary/HBoxContainer/TotalEnergyLabel
	
	if rows_data != null:
		for row_data in rows_data:
			add_row({ 
				"building_id": row_data.building_id,
				"building_count": row_data.building_count
			})
	
	if table_rows.size() < initial_table_rows:
		add_row({})
	
	table_title_line_edit.text = title
	if title_placeholder != null:
		table_title_line_edit.placeholder_text = title_placeholder


func _ready():
	render_total_energy_label()


func _on_total_energy_changed(_value: int):
	render_total_energy_label()
	changed.emit()


func _on_add_building_button_pressed():
	add_row({})


func _on_table_title_line_edit_text_changed(new_text):
	title = new_text
	changed.emit()


func get_save_data() -> Dictionary:
	var rows_json_data = table_rows.map(func get_row(row): return row.get_save_data())
	
	return {
		"title": title,
		"title_placeholder": title_placeholder,
		"rows": rows_json_data,
	}

